/*
    This file is part of Kig, a KDE program for Interactive Geometry.
    SPDX-FileCopyrightText: 2002 Dominique Devriese <devriese@kde.org>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "kig_iface.h"

KigIface::KigIface()
    : DCOPObject("KigIface")
{
}

KigIface::~KigIface()
{
}
